@@ -1,17 +1,19 @@
 (ns ring.middleware.resource
   "Middleware for serving static resources."
-  (require [ring.util.codec :as codec]
-           [ring.util.response :as response]
-           [ring.util.request :as request]
-           [ring.middleware.head :as head]))
+  (:require [ring.util.codec :as codec]
+            [ring.util.response :as response]
+            [ring.util.request :as request]
+            [ring.middleware.head :as head]))
 
 (defn resource-request
   "If request matches a static resource, returns it in a response map.
- Otherwise returns nil."
+  Otherwise returns nil."
+  {:added "1.2"}
   [request root-path]
-  (if (= :get (:request-method request))
+  (if (#{:head :get} (:request-method request))
     (let [path (subs (codec/url-decode (request/path-info request)) 1)]
-      (response/resource-response path {:root root-path}))))
+      (-> (response/resource-response path {:root root-path})
+          (head/head-response request)))))
 
 (defn wrap-resource
   "Middleware that first checks to see whether the request map matches a static
@@ -20,5 +22,5 @@
   added to the beginning of the resource path."
   [handler root-path]
   (fn [request]
-    (or ((head/wrap-head #(resource-request % root-path)) request)
+    (or (resource-request request root-path)
         (handler request))))
