@@ -9,6 +9,15 @@ SyncHttpClient = com.puppetlabs.http.client.Sync
 java_import com.puppetlabs.http.client.SimpleRequestOptions
 java_import com.puppetlabs.http.client.ResponseBodyType
 
+class Puppet::Server::HttpClientError < SocketError
+  attr_reader :cause
+
+  def initialize(message, cause = nil)
+    super(message)
+    @cause = cause
+  end
+end
+
 class Puppet::Server::HttpClient
 
   OPTION_DEFAULTS = {
@@ -20,7 +29,10 @@ class Puppet::Server::HttpClient
   # Store a java HashMap of settings related to the http client
   def self.initialize_settings(settings)
     @settings = settings.select { |k,v|
-      ["ssl_protocols", "cipher_suites"].include? k
+      ["ssl_protocols",
+       "cipher_suites",
+       "http_connect_timeout_milliseconds",
+       "http_idle_timeout_milliseconds"].include? k
     }
   end
 
@@ -47,16 +59,17 @@ class Puppet::Server::HttpClient
       end
 
       # http://en.wikipedia.org/wiki/Basic_access_authentication#Client_side
-      headers["Authorization"] =
-          "Basic #{Base64.strict_encode64 "#{credentials[:user]}:#{credentials[:password]}"}"
+      encoded = Base64.strict_encode64("#{credentials[:user]}:#{credentials[:password]}")
+      headers["Authorization"] = "Basic #{encoded}"
     end
 
     request_options = SimpleRequestOptions.new(build_url(url))
     request_options.set_headers(headers)
     request_options.set_as(ResponseBodyType::TEXT)
     request_options.set_body(body)
+    configure_timeouts(request_options)
     configure_ssl(request_options)
-    response = SyncHttpClient.post(request_options)
+    response = client_post(request_options)
     ruby_response(response)
   end
 
@@ -64,13 +77,38 @@ class Puppet::Server::HttpClient
     request_options = SimpleRequestOptions.new(build_url(url))
     request_options.set_headers(headers)
     request_options.set_as(ResponseBodyType::TEXT)
+    configure_timeouts(request_options)
     configure_ssl(request_options)
-    response = SyncHttpClient.get(request_options)
+    response = client_get(request_options)
     ruby_response(response)
   end
 
   private
 
+  def client_get(request_options)
+    SyncHttpClient.get(request_options)
+  rescue Java::ComPuppetlabsHttpClient::HttpClientException => e
+    raise Puppet::Server::HttpClientError.new(e.message, e)
+  end
+
+  def client_post(request_options)
+    SyncHttpClient.post(request_options)
+  rescue Java::ComPuppetlabsHttpClient::HttpClientException => e
+    raise Puppet::Server::HttpClientError.new(e.message, e)
+  end
+
+  def configure_timeouts(request_options)
+    settings = self.class.settings
+
+    if settings.has_key?("http_connect_timeout_milliseconds")
+      request_options.set_connect_timeout_milliseconds(settings["http_connect_timeout_milliseconds"])
+    end
+
+    if settings.has_key?("http_idle_timeout_milliseconds")
+      request_options.set_socket_timeout_milliseconds(settings["http_idle_timeout_milliseconds"])
+    end
+  end
+
   def configure_ssl(request_options)
     return unless @use_ssl
     request_options.set_ssl_context(Puppet::Server::Config.ssl_context)
@@ -116,4 +154,4 @@ class Puppet::Server::HttpClient
     result
   end
 
-end
\ No newline at end of file
+end
