@@ -8,7 +8,8 @@
             [clojure.java.io :as io]
             [clojure.tools.logging :as log])
   (:import (puppetlabs.services.jruby.jruby_puppet_schemas JRubyPuppetInstance)
-           (com.puppetlabs.puppetserver PuppetProfiler)))
+           (com.puppetlabs.puppetserver PuppetProfiler)
+           (clojure.lang IFn)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Constants
@@ -51,9 +52,10 @@
   (jruby-internal/get-pool context))
 
 (schema/defn ^:always-validate
-  pool->vec :- [JRubyPuppetInstance]
+  registered-instances :- [JRubyPuppetInstance]
   [context :- jruby-schemas/PoolContext]
   (-> (get-pool context)
+      .getRegisteredElements
       .iterator
       iterator-seq
       vec))
@@ -65,6 +67,54 @@
     (throw (IllegalArgumentException. (str "No configuration data found.  Perhaps "
                                            "you did not specify the --config option?")))))
 
+(schema/defn create-requested-event :- jruby-schemas/JRubyRequestedEvent
+  [reason :- jruby-schemas/JRubyEventReason]
+  {:type :instance-requested
+   :reason reason})
+
+(schema/defn create-borrowed-event :- jruby-schemas/JRubyBorrowedEvent
+  [requested-event :- jruby-schemas/JRubyRequestedEvent
+   instance :- jruby-schemas/JRubyPuppetBorrowResult]
+  {:type :instance-borrowed
+   :reason (:reason requested-event)
+   :requested-event requested-event
+   :instance instance})
+
+(schema/defn create-returned-event :- jruby-schemas/JRubyReturnedEvent
+  [instance :- jruby-schemas/JRubyPuppetInstanceOrRetry
+   reason :- jruby-schemas/JRubyEventReason]
+  {:type :instance-returned
+   :reason reason
+   :instance instance})
+
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;; Support functions for event notification
+
+(schema/defn notify-event-listeners :- jruby-schemas/JRubyEvent
+  [event-callbacks :- [IFn]
+   event :- jruby-schemas/JRubyEvent]
+  (doseq [f event-callbacks]
+    (f event))
+  event)
+
+(schema/defn instance-requested :- jruby-schemas/JRubyRequestedEvent
+  [event-callbacks :- [IFn]
+   reason :- jruby-schemas/JRubyEventReason]
+  (notify-event-listeners event-callbacks (create-requested-event reason)))
+
+(schema/defn instance-borrowed :- jruby-schemas/JRubyBorrowedEvent
+  [event-callbacks :- [IFn]
+   requested-event :- jruby-schemas/JRubyRequestedEvent
+   instance :- jruby-schemas/JRubyPuppetBorrowResult]
+  (notify-event-listeners event-callbacks (create-borrowed-event requested-event instance)))
+
+(schema/defn instance-returned :- jruby-schemas/JRubyReturnedEvent
+  [event-callbacks :- [IFn]
+   instance :- jruby-schemas/JRubyPuppetInstanceOrRetry
+   reason :- jruby-schemas/JRubyEventReason]
+  (notify-event-listeners event-callbacks (create-returned-event instance reason)))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Public
 
@@ -119,7 +169,7 @@
   mark-environment-expired!
   [context :- jruby-schemas/PoolContext
    env-name :- schema/Str]
-  (doseq [jruby-instance (pool->vec context)]
+  (doseq [jruby-instance (registered-instances context)]
     (-> jruby-instance
       :environment-registry
       (puppet-env/mark-environment-expired! env-name))))
@@ -127,7 +177,7 @@
 (schema/defn ^:always-validate
   mark-all-environments-expired!
   [context :- jruby-schemas/PoolContext]
-  (doseq [jruby-instance (pool->vec context)]
+  (doseq [jruby-instance (registered-instances context)]
     (-> jruby-instance
         :environment-registry
         puppet-env/mark-all-environments-expired!)))
@@ -136,12 +186,16 @@
   borrow-from-pool :- jruby-schemas/JRubyPuppetInstanceOrRetry
   "Borrows a JRubyPuppet interpreter from the pool. If there are no instances
   left in the pool then this function will block until there is one available."
-  [pool-context :- jruby-schemas/PoolContext]
-  (jruby-internal/borrow-from-pool
-    pool-context))
+  [pool-context :- jruby-schemas/PoolContext
+   reason :- schema/Any
+   event-callbacks :- [IFn]]
+  (let [requested-event (instance-requested event-callbacks reason)
+        instance (jruby-internal/borrow-from-pool pool-context)]
+    (instance-borrowed event-callbacks requested-event instance)
+    instance))
 
 (schema/defn ^:always-validate
-  borrow-from-pool-with-timeout :- (schema/maybe jruby-schemas/JRubyPuppetInstanceOrRetry)
+  borrow-from-pool-with-timeout :- jruby-schemas/JRubyPuppetBorrowResult
   "Borrows a JRubyPuppet interpreter from the pool, like borrow-from-pool but a
   blocking timeout is provided. If an instance is available then it will be
   immediately returned to the caller, if not then this function will block
@@ -149,16 +203,24 @@
   timeout. If the timeout runs out then nil will be returned, indicating that
   there were no instances available."
   [pool-context :- jruby-schemas/PoolContext
-   timeout :- schema/Int]
+   timeout :- schema/Int
+   reason :- schema/Any
+   event-callbacks :- [IFn]]
   {:pre  [(>= timeout 0)]}
-  (jruby-internal/borrow-from-pool-with-timeout
-    pool-context
-    timeout))
+  (let [requested-event (instance-requested event-callbacks reason)
+        instance (jruby-internal/borrow-from-pool-with-timeout
+                   pool-context
+                   timeout)]
+    (instance-borrowed event-callbacks requested-event instance)
+    instance))
 
 (schema/defn ^:always-validate
   return-to-pool
   "Return a borrowed pool instance to its free pool."
-  [instance :- jruby-schemas/JRubyPuppetInstanceOrRetry]
+  [instance :- jruby-schemas/JRubyPuppetInstanceOrRetry
+   reason :- schema/Any
+   event-callbacks :- [IFn]]
+  (instance-returned event-callbacks instance reason)
   (jruby-internal/return-to-pool instance))
 
 (schema/defn ^:always-validate cli-ruby! :- jruby-schemas/JRubyMainStatus
